@@ -4,6 +4,7 @@ import json
 import logging
 import os
 import pathlib
+from dataclasses import dataclass, field
 from itertools import chain
 from typing import List, Union
 
@@ -18,6 +19,7 @@ from ada.concepts.containers import (
     Sections,
 )
 from ada.concepts.piping import Pipe
+from ada.concepts.points import Node
 from ada.concepts.primitives import (
     Penetration,
     PrimBox,
@@ -28,23 +30,30 @@ from ada.concepts.primitives import (
 )
 from ada.concepts.structural import Beam, Material, Plate, Section, Wall
 from ada.config import Settings, User
-from ada.fem import FEM
+from ada.fem import (
+    Amplitude,
+    Bc,
+    Connector,
+    ConnectorSection,
+    Constraint,
+    Csys,
+    Elem,
+    FemSection,
+    FemSet,
+    Interaction,
+    InteractionProperty,
+    Mass,
+    PredefinedField,
+    Spring,
+    Step,
+    Surface,
+)
+from ada.fem.containers import FemElements, FemSections, FemSets
 from ada.ifc.utils import create_guid
 
 
 class Part(BackendGeom):
-    """
-    A Part superclass design to host all relevant information for cad and FEM modelling.
-
-    :param name: Name of Part
-    :param colour: Colour of Part.
-    :param origin: Origin of part.
-    :param lx: Local X
-    :param ly: Local Y
-    :param lz: Local Z
-    :param settings: A properties object
-    :param metadata: A dict for containing metadata
-    """
+    """A Part superclass design to host all relevant information for cad and FEM modelling."""
 
     def __init__(
         self,
@@ -92,7 +101,7 @@ class Part(BackendGeom):
         if fem is not None:
             fem.parent = self
 
-        self._fem = FEM(name + "-1", parent=self) if fem is None else fem
+        self.fem = FEM(name + "-1", parent=self) if fem is None else fem
 
     def add_beam(self, beam: Beam) -> Beam:
         if beam.units != self.units:
@@ -200,7 +209,9 @@ class Part(BackendGeom):
             section.units = self.units
         return self._sections.add(section)
 
-    def add_penetration(self, pen: Penetration, add_pen_to_subparts=True) -> Penetration:
+    def add_penetration(
+        self, pen: Union[Penetration, PrimExtrude, PrimRevolve, PrimCyl, PrimBox], add_pen_to_subparts=True
+    ) -> Penetration:
         if type(pen) in (PrimExtrude, PrimRevolve, PrimCyl, PrimBox):
             pen = Penetration(pen, parent=self)
 
@@ -275,7 +286,7 @@ class Part(BackendGeom):
         :param opacity: Assign Opacity upon import
         :param source_units: Unit of the imported STEP file. Default is 'm'
         """
-        from ..occ.utils import extract_shapes
+        from ada.occ.utils import extract_shapes
 
         shapes = extract_shapes(step_path, scale, transform, rotate)
         if len(shapes) > 0:
@@ -300,13 +311,8 @@ class Part(BackendGeom):
     def get_part(self, name) -> Part:
         return self.parts[name]
 
-    def get_by_name(self, name):
-        """
-        Get element of any type by its name.
-
-        :param name:
-        :return:
-        """
+    def get_by_name(self, name) -> Union[Part, Plate, Beam, Shape, Material, None]:
+        """Get element of any type by its name."""
         for p in self.get_all_subparts() + [self]:
             if p.name == name:
                 return p
@@ -472,13 +478,14 @@ class Part(BackendGeom):
     def fem(self) -> FEM:
         return self._fem
 
+    @fem.setter
+    def fem(self, value: FEM):
+        value.parent = self
+        self._fem = value
+
     @property
     def gmsh(self):
-        """
-
-        :return:
-        :rtype: ada.fem.io.mesh.GMesh
-        """
+        """:rtype: ada.fem.mesh.GMesh"""
         return self._gmsh
 
     @property
@@ -493,23 +500,14 @@ class Part(BackendGeom):
     def materials(self) -> Materials:
         return self._materials
 
-    @property
-    def bbox(self):
-        if len(self.fem.nodes) != 0:
-            return self.fem.nodes.bbox
-        elif len(self.nodes) != 0:
-            return self.nodes.bbox
-        else:
-            raise ValueError("This part contains no Nodes")
-
     @property
     def colour(self):
         if self._colour is None:
             from random import randint
 
-            return randint(0, 255) / 255, randint(0, 255) / 255, randint(0, 255) / 255
-        else:
-            return self._colour
+            self._colour = randint(0, 255) / 255, randint(0, 255) / 255, randint(0, 255) / 255
+
+        return self._colour
 
     @colour.setter
     def colour(self, value):
@@ -612,11 +610,7 @@ class Part(BackendGeom):
 
 
 class Assembly(Part):
-    """
-    The Assembly object. A top level container of parts, beams, plates, shapes and FEM.
-
-
-    """
+    """The Assembly object. A top level container of parts, beams, plates, shapes and FEM."""
 
     def __init__(
         self,
@@ -771,13 +765,12 @@ class Assembly(Part):
         :param elements2part: Grab all physical elements from ifc and import it to the parsed in Part object.
         :param cache_model_now:
         """
-        import ifcopenshell
-
         from ada.ifc.utils import (
             add_to_assembly,
             get_parent,
             import_ifc_hierarchy,
             import_physical_ifc_elem,
+            open_ifc,
             scale_ifc_file,
         )
 
@@ -785,7 +778,7 @@ class Assembly(Part):
             if self._from_cache(ifc_file) is True:
                 return None
 
-        f = ifcopenshell.open(ifc_file)
+        f = open_ifc(ifc_file)
 
         scaled_ifc = scale_ifc_file(self.ifc_file, f)
         if scaled_ifc is not None:
@@ -819,7 +812,12 @@ class Assembly(Part):
             self._to_cache(ifc_file, cache_model_now)
 
     def read_fem(
-        self, fem_file: os.PathLike, fem_format=None, name=None, fem_converter="default", cache_model_now=False
+        self,
+        fem_file: Union[str, os.PathLike],
+        fem_format: str = None,
+        name: str = None,
+        fem_converter="default",
+        cache_model_now=False,
     ):
         """
         Import a Finite Element model.
@@ -831,9 +829,6 @@ class Assembly(Part):
         :param name:
         :param fem_converter: Set desired fem converter. Use either 'default' or 'meshio'.
         :param cache_model_now:
-        :type fem_file: Union[str, os.PathLike]
-        :type fem_format: str
-        :type name: str
 
         Note! The meshio fem converter implementation currently only supports reading elements and nodes.
         """
@@ -897,7 +892,7 @@ class Assembly(Part):
         :param overwrite: Overwrite existing input file deck
         :param fem_converter: Set desired fem converter. Use either 'default' or 'meshio'.
         :param exit_on_complete:
-        :rtype: ada.base.render_fem.Results
+        :rtype: ada.fem.results.Results
 
             Note! Meshio implementation currently only supports reading & writing elements and nodes.
 
@@ -912,9 +907,14 @@ class Assembly(Part):
         """
         from ada.fem.io import fem_executables, get_fem_converters
         from ada.fem.io.utils import folder_prep, should_convert
+        from ada.fem.results import Results
 
         base_path = Settings.scratch_dir / name / name
-        fem_res_files = dict(code_aster=base_path.with_suffix(".rmed"), abaqus=base_path.with_suffix(".odb"))
+        fem_res_files = dict(
+            code_aster=base_path.with_suffix(".rmed"),
+            abaqus=base_path.with_suffix(".odb"),
+            calculix=base_path.with_suffix(".frd"),
+        )
         res_path = fem_res_files.get(fem_format, None)
         metadata = dict() if metadata is None else metadata
         metadata["fem_format"] = fem_format
@@ -923,6 +923,7 @@ class Assembly(Part):
         if should_convert(res_path, overwrite):
             analysis_dir = folder_prep(scratch_dir, name, overwrite)
             _, fem_exporter = get_fem_converters("", fem_format, fem_converter)
+
             if fem_exporter is None:
                 raise ValueError(f'FEM export for "{fem_format}" using "{fem_converter}" is currently not supported')
             fem_inp_files = dict(
@@ -949,17 +950,9 @@ class Assembly(Part):
                 )
         else:
             print(f'Result file "{res_path}" already exists.\nUse "overwrite=True" if you wish to overwrite')
-
-        if fem_format not in fem_res_files.keys():
-            logging.info(f'FEM format "{fem_format}" is not yet supported for results processing')
+        if out is None and res_path is None:
             return None
-
-        if res_path.exists():
-            from ada.fem.results import Results
-
-            return Results(res_path, output=out)
-        else:
-            logging.info(f'Result file "{res_path}" was not found')
+        return Results(res_path, output=out)
 
     def to_ifc(self, destination_file, include_fem=False) -> None:
         from ada.ifc.export import add_part_objects_to_ifc
@@ -968,7 +961,7 @@ class Assembly(Part):
 
         dest = pathlib.Path(destination_file).with_suffix(".ifc")
 
-        for s in self.sections.nmap.values():
+        for s in self.sections:
             f.add(s.ifc_profile)
             f.add(s.ifc_beam_type)
 
@@ -1060,15 +1053,11 @@ class Assembly(Part):
         return site
 
     def get_ifc_source_by_name(self, ifc_file):
-        """
-
-        :param ifc_file:
-        :return:
-        """
-        import ifcopenshell
+        from ada.ifc.utils import open_ifc
 
         if ifc_file not in self._source_ifc_files.keys():
-            ifc_f = ifcopenshell.open(ifc_file)
+
+            ifc_f = open_ifc(ifc_file)
             self._source_ifc_files[ifc_file] = ifc_f
         else:
             ifc_f = self._source_ifc_files[ifc_file]
@@ -1094,7 +1083,7 @@ class Assembly(Part):
     def ifc_sections(self):
         if self._ifc_sections is None:
             secrel = dict()
-            for sec in self.sections.nmap.values():
+            for sec in self.sections:
                 secrel[sec.name] = sec.ifc_profile, sec.ifc_beam_type
             self._ifc_sections = secrel
         return self._ifc_sections
@@ -1131,3 +1120,260 @@ class Assembly(Part):
             f'Assembly("{self.name}": Beams: {nbms}, Plates: {npls}, Pipes: {npipes}, '
             f"Shapes: {nshps}, Elements: {nels}, Nodes: {nns})"
         )
+
+
+@dataclass
+class FEM:
+    name: str
+    metadata: dict = field(default_factory=dict)
+    parent: Part = field(init=True, default=None)
+
+    masses: dict[str, Mass] = field(init=False, default_factory=dict)
+    surfaces: dict[str, Surface] = field(init=False, default_factory=dict)
+    amplitudes: dict[str, Amplitude] = field(init=False, default_factory=dict)
+    connectors: dict[str, Connector] = field(init=False, default_factory=dict)
+    connector_sections: dict[str, ConnectorSection] = field(init=False, default_factory=dict)
+    springs: dict[str, Spring] = field(init=False, default_factory=dict)
+    intprops: dict[str, InteractionProperty] = field(init=False, default_factory=dict)
+    interactions: dict[str, Interaction] = field(init=False, default_factory=dict)
+    predefined_fields: dict[str, PredefinedField] = field(init=False, default_factory=dict)
+    lcsys: dict[str, Csys] = field(init=False, default_factory=dict)
+
+    bcs: List[Bc] = field(init=False, default_factory=list)
+    constraints: List[Constraint] = field(init=False, default_factory=list)
+    steps: List[Step] = field(init=False, default_factory=list)
+
+    nodes: Nodes = field(default_factory=Nodes, init=True)
+    elements: FemElements = field(default_factory=FemElements, init=True)
+    sets: FemSets = field(default_factory=FemSets, init=True)
+    sections: FemSections = field(default_factory=FemSections, init=True)
+    initial_state: PredefinedField = field(default=None, init=True)
+    subroutine: str = field(default=None, init=True)
+
+    def __post_init__(self):
+        self.nodes.parent = self
+        self.elements.parent = self
+        self.sets.parent = self
+        self.sections.parent = self
+
+    def add_elem(self, elem: Elem):
+        elem.parent = self
+        self.elements.add(elem)
+
+    def add_section(self, section: FemSection):
+        section.parent = self
+        self.sections.add(section)
+
+    def add_bc(self, bc: Bc):
+        if bc.name in [b.name for b in self.bcs]:
+            raise Exception('BC with name "{bc_id}" already exists'.format(bc_id=bc.name))
+
+        bc.parent = self
+        if bc.fem_set.parent is None:
+            # TODO: look over this implementation. Is this okay?
+            logging.debug("Bc FemSet has no parent. Adding to self")
+            self.sets.add(bc.fem_set)
+
+        self.bcs.append(bc)
+
+    def add_mass(self, mass: Mass):
+        mass.parent = self
+        self.masses[mass.name] = mass
+
+    def add_set(
+        self,
+        fem_set: FemSet,
+        ids=None,
+        p=None,
+        vol_box=None,
+        vol_cyl=None,
+        single_member=False,
+        tol=1e-4,
+    ) -> FemSet:
+        """
+        Simple method that creates a set string based on a set name, node or element ids and adds it to the assembly str
+
+        :param fem_set: A fem set object
+        :param ids: List of integers
+        :param p: Single point (x,y,z)
+        :param vol_box: Search by quadratic volume. Where p is (xmin, ymin, zmin) and vol_box is (xmax, ymax, zmax)
+        :param vol_cyl: Search by cylindrical volume. Used together with p to find
+                        nodes within cylinder inputted by [radius, height, thickness]
+        :param single_member: Set True if you wish to keep only a single member
+        :param tol: Point Tolerances. Default is 1e-4
+        """
+        if ids is not None:
+            fem_set.add_members(ids)
+
+        fem_set.parent = self
+
+        def append_members(nodelist):
+            if single_member is True:
+                fem_set.add_members([nodelist[0]])
+            else:
+                fem_set.add_members(nodelist)
+
+        if fem_set.type == "nset":
+            if p is not None or vol_box is not None or vol_cyl is not None:
+                nodes = self.nodes.get_by_volume(p, vol_box, vol_cyl, tol)
+                if len(nodes) == 0 and self.parent is not None:
+                    assembly = self.parent.get_assembly()
+                    list_of_ps = assembly.get_all_subparts() + [assembly]
+                    for part in list_of_ps:
+                        nodes = part.fem.nodes.get_by_volume(p, vol_box, vol_cyl, tol)
+                        if len(nodes) > 0:
+                            fem_set.parent = part.fem
+                            append_members(nodes)
+                            part.fem.add_set(fem_set)
+                            return fem_set
+
+                    raise Exception(f'No nodes found for fem set "{fem_set.name}"')
+                elif nodes is not None and len(nodes) > 0:
+                    append_members(nodes)
+                else:
+                    raise Exception(f'No nodes found for femset "{fem_set.name}"')
+
+        self.sets.add(fem_set)
+        return fem_set
+
+    def add_step(self, step: Step) -> Step:
+        """Add an analysis step to the assembly"""
+        if len(self.steps) > 0:
+            if self.steps[-1].type != "eigenfrequency" and step.type == "complex_eig":
+                raise Exception(
+                    "complex eigenfrequency analysis step needs to follow eigenfrequency step. Check your input"
+                )
+        step.parent = self
+        self.steps.append(step)
+
+        return step
+
+    def add_interaction_property(self, int_prop: InteractionProperty) -> InteractionProperty:
+        int_prop.parent = self
+        self.intprops[int_prop.name] = int_prop
+        return int_prop
+
+    def add_interaction(self, interaction: Interaction) -> Interaction:
+        interaction.parent = self
+        self.interactions[interaction.name] = interaction
+        return interaction
+
+    def add_constraint(self, constraint: Constraint) -> Constraint:
+        constraint.parent = self
+        self.constraints.append(constraint)
+        return constraint
+
+    def add_lcsys(self, lcsys: Csys) -> Csys:
+        if lcsys.name in self.lcsys.keys():
+            raise ValueError("Local Coordinate system cannot have duplicate name")
+        lcsys.parent = self
+        self.lcsys[lcsys.name] = lcsys
+        return lcsys
+
+    def add_connector_section(self, connector_section: ConnectorSection):
+        connector_section.parent = self
+        self.connector_sections[connector_section.name] = connector_section
+
+    def add_connector(self, connector: Connector):
+        connector.parent = self
+        self.connectors[connector.name] = connector
+        connector.csys.parent = self
+        self.elements.add(connector)
+        self.add_set(FemSet(name=connector.name, members=[connector.id], set_type="elset"))
+
+    def add_rp(self, name, node: Node):
+        """Adds a reference point in assembly with a specific name"""
+        node.parent = self
+        self.nodes.add(node)
+        fem_set = self.add_set(FemSet(name, [node], "nset"))
+        return node, fem_set
+
+    def add_surface(self, surface: Surface):
+        surface.parent = self
+        self.surfaces[surface.name] = surface
+
+    def add_amplitude(self, amplitude: Amplitude):
+        amplitude.parent = self
+        self.amplitudes[amplitude.name] = amplitude
+
+    def add_predefined_field(self, pre_field: PredefinedField):
+        pre_field.parent = self
+
+        self.predefined_fields[pre_field.name] = pre_field
+
+    def add_spring(self, spring: Spring):
+        # self.elements.add(spring)
+
+        if spring.fem_set.parent is None:
+            self.sets.add(spring.fem_set)
+        self.springs[spring.name] = spring
+
+    def create_fem_elem_from_obj(self, obj, el_type=None) -> Elem:
+        """Converts structural object to FEM elements. Currently only BEAM is supported"""
+        from ada.fem.shapes import ElemType
+
+        if type(obj) is not Beam:
+            raise NotImplementedError(f'Object type "{type(obj)}" is not yet supported')
+
+        el_type = "B31" if el_type is None else el_type
+
+        res = self.nodes.add(obj.n1)
+        if res is not None:
+            obj.n1 = res
+        res = self.nodes.add(obj.n2)
+        if res is not None:
+            obj.n2 = res
+
+        elem = Elem(None, [obj.n1, obj.n2], el_type)
+        self.add_elem(elem)
+        femset = FemSet(f"{obj.name}_set", [elem.id], "elset")
+        self.add_set(femset)
+        self.add_section(
+            FemSection(
+                f"d{obj.name}_sec",
+                ElemType.LINE,
+                femset,
+                obj.material,
+                obj.section,
+                obj.ori[1],
+            )
+        )
+        return elem
+
+    @property
+    def instance_name(self):
+        return self.name if self.name is not None else f"{self.parent.name}-1"
+
+    @property
+    def nsets(self):
+        return self.sets.nodes
+
+    @property
+    def elsets(self):
+        return self.sets.elements
+
+    def __add__(self, other: FEM):
+        # Nodes
+        nodid_max = self.nodes.max_nid if len(self.nodes) > 0 else 0
+        if nodid_max > other.nodes.min_nid:
+            other.nodes.renumber(int(nodid_max + 10))
+
+        self.nodes += other.nodes
+
+        # Elements
+        elid_max = self.elements.max_el_id if len(self.elements) > 0 else 0
+
+        if elid_max > other.elements.min_el_id:
+            other.elements.renumber(int(elid_max + 10))
+
+        logging.info("FEM operand type += is still ")
+
+        self.elements += other.elements
+        self.sections += other.sections
+        self.sets += other.sets
+        self.lcsys.update(other.lcsys)
+
+        return self
+
+    def __repr__(self):
+        return f"FEM({self.name}, Elements: {len(self.elements)}, Nodes: {len(self.nodes)})"
